/***************************************************************************
 *   Copyright (C) 2006 by jkeller   *
 *   jkeller@localhost   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef __MATRIX_HXX__
#define __MATRIX_HXX__

#include <iostream>
#include <utility> //for pair<T,U>()
#include <cmath>   //for sqrt()
#include <exception>
using namespace std;

#ifdef USE_ALTIVEC
#include <vecLib/vDSP.h>
#endif

class BadFFT : public exception
{
  public:
    virtual ~BadFFT() throw() {};
    virtual const char* what(void) const throw()
            {return "Bad FFT : input and output matrices should have same size,\n"
	            "which should be a power of 2";}
};

//class MMatrix
template<typename T> class MMatrix;
template<typename T> ostream& operator<<(ostream&,const MMatrix<T>&);
template<typename T> istream& operator>>(istream&,MMatrix<T>&);

template<typename T = float>
class MMatrix
{
  public:
    //Constructors
    MMatrix(size_t aHeight=0, size_t aWidth=0, const T& fill=T(),bool aBinaryMode = true)
	 :height(aHeight),width(aWidth),data(0),binaryMode(aBinaryMode)
	 {data = allocation(height,width,fill);}

    MMatrix(const MMatrix<T>& other)
         :height(other.height),width(other.width),data(0),binaryMode(other.binaryMode)
         {data=allocation(height,width);copy(other);}
    template<typename U> MMatrix(const MMatrix<U>& other)
                              :height(other.getHeight()),width(other.getWidth()),data(0),
                               binaryMode(other.getBinaryMode())
			 {data=allocation(height,width);copy(other);}
    
    template <typename U>
    MMatrix(const MMatrix<U>& other, size_t fromRow, size_t fromCol)
         :height(other.getHeight()-fromRow),width(other.getWidth()-fromCol),data(0),
          binaryMode(other.getBinaryMode())
	 {data=allocation(height,width);
          copy(other, fromRow, fromCol, height, width);}
    
    template <typename U>
    MMatrix(const MMatrix<U>& other, size_t fromRow, size_t fromCol,
                                   size_t aHeight, size_t aWidth,
				   const T& fill = T(), bool loop = false)
         :height(aHeight),width(aWidth),data(0),binaryMode(other.getBinaryMode())
	 {data=allocation(height,width,fill);
	  copy(other, fromRow, fromCol, height, width, loop);}
    
    //operator=
    MMatrix<T>& operator=(const MMatrix<T>& other)
	       {if (((void*)&other) != ((void*)this))
		   {clean();height=other.height;width=other.width;
		    data=allocation(height,width);copy(other);} return *this;}
    template<typename U> MMatrix<T>& operator=(const MMatrix<U>& other)
	       {if (((void*)&other) != ((void*)this))
		   {clean();height=other.getHeight();width=other.getWidth();
		    data=allocation(height,width);copy(other);} return *this;}
    
    //destructor
    virtual ~MMatrix() {clean();}

  public:
    template<typename U> operator MMatrix<U>() const;

  public:
    T*        operator[](size_t index)       {return data[index];}
    const T*  operator[](size_t index) const {return data[index];}
    
  public:
//    friend class <> MMatrix;
    template<typename U> MMatrix<T>& operator+=(const MMatrix<U>& other);
    template<typename U> MMatrix<T>& operator-=(const MMatrix<U>& other);
    template<typename U> MMatrix<T>& operator*=(const MMatrix<U>& other);
    template<typename U> MMatrix<T>& operator/=(const MMatrix<U>& other);
    template<typename U> MMatrix<T>& operator+=(const U& x);
    template<typename U> MMatrix<T>& operator-=(const U& x);
    template<typename U> MMatrix<T>& operator*=(const U& x);
    template<typename U> MMatrix<T>& operator/=(const U& x);
    template<typename U> MMatrix<T>& operator&=(const MMatrix<U>& other);
    template<typename U> MMatrix<T>& operator|=(const MMatrix<U>& other);
    template<typename U> MMatrix<T>& operator^=(const MMatrix<U>& other);
    template<typename U> MMatrix<T>& operator&=(const U& x);
    template<typename U> MMatrix<T>& operator|=(const U& x);
    template<typename U> MMatrix<T>& operator^=(const U& x);
    MMatrix<T>  operator~(void);
    MMatrix<T>& operator++(void) {return (*this)+=static_cast<T>(1);}
    MMatrix<T>  operator++(int) {MMatrix<T> save(*this); ++(*this); return save;}
    MMatrix<T>& operator--(void) {return (*this)-=static_cast<T>(1);}
    MMatrix<T>  operator--(int) {MMatrix<T> save(*this); --(*this); return save;}
    MMatrix<T> operator-() const {MMatrix<T> m(*this); return m.oppose();}
    T sum(void) const;
    T product(void) const;

    MMatrix<T>& transpose(void);

  public:
    //resize
    MMatrix<T>& resize(size_t height, size_t width, const T& fill=T())
               {return resize(height,width,0,0,fill);}
    MMatrix<T>& resize(size_t height, size_t width,
		      size_t topMargin, size_t leftMargin, const T& fill=T());

    //subMMatrix
    MMatrix<T>  subMMatrix(size_t fromRow, size_t fromCol,
                         bool loop = false) const
	       {if (loop) {if (height) fromRow%=height; if (width) fromCol%=width;}
				  return subMMatrix(fromRow,fromCol,
			                     height>fromRow ? height-fromRow : 0,
			                     width >fromCol ? width-fromCol  : 0,
													 T(),loop);}
    MMatrix<T>  subMMatrix(size_t fromRow, size_t fromCol,
		         size_t height,  size_t width,
			 const T& fill = T(), bool loop = false) const
	       {return MMatrix(*this,fromRow,fromCol,height,width,fill,loop);}
    
    //fillWith
    MMatrix<T>& reset(const T& fill = T()) {return fillWith(fill);}
    MMatrix<T>& fillWith(const T& fill) {if (data) memorySet(data[0],height*width,fill);return *this;}
    
    template<typename U> MMatrix<T>& fillWith(const MMatrix<U>& other, bool loop = false)
            {return fillWith(other,0,0,loop);}
    template<typename U> MMatrix<T>& fillWith(const MMatrix<U>& other,
		                             size_t fromRow, size_t fromCol,
					     bool loop = false)
            {return fillWith(0,0,other,fromRow,fromCol,height,width,loop);}
    
    template<typename U> MMatrix<T>& fillWith(const MMatrix<U>& other,
		                             size_t fromRow, size_t fromCol,
					     size_t height,  size_t width,
					     bool loop = false)
            {return fillWith(0,0,other,fromRow,fromCol,height,width,loop);}
    
    MMatrix<T>& fillWith(size_t targetRow, size_t targetCol,
		        const MMatrix<T>& other, bool loop = false)
	       {return fillWith(targetRow,targetCol,other,0,0,loop);}
    
    template <typename U>
    MMatrix<T>& fillWith(size_t targetRow, size_t targetCol,
		        const MMatrix<U>& other, bool loop = false)
	       {return fillWith(targetRow,targetCol,other,0,0,loop);}

    MMatrix<T>& fillWith(size_t targetRow, size_t targetCol,
		        const MMatrix<T>& other,
		        size_t fromRow=0, size_t fromCol=0,bool loop = false)
	       {return fillWith(targetRow,targetCol,other,fromRow,fromCol,
			        loop ? (height) :
				       ((other.getHeight()>fromRow) ? other.getHeight()-fromRow : 0),
			        loop ? (width) :
				       ((other.getWidth()>fromCol) ? other.getWidth()-fromCol : 0),
				loop);}
    
    template<typename U> MMatrix<T>& fillWith(size_t targetRow, size_t targetCol,
		                             const MMatrix<U>& other,
		                             size_t fromRow=0, size_t fromCol=0,
					     bool loop = false)
	       {return fillWith(targetRow,targetCol,other,fromRow,fromCol,
			        loop ? (height) :
				       ((other.getHeight()>fromRow) ? other.getHeight()-fromRow : 0),
			        loop ? (width) :
				       ((other.getWidth()>fromCol) ? other.getWidth()-fromCol : 0),
				loop);}

    MMatrix<T>& fillWith(size_t targetRow, size_t targetCol,
		        const MMatrix<T>& other,
		        size_t fromRow, size_t fromCol,
			size_t height,  size_t width,
			bool loop = false);
    
    template<typename U> MMatrix<T>& fillWith(size_t targetRow, size_t targetCol,
		                             const MMatrix<U>& other,
		                             size_t fromRow, size_t fromCol,
					     size_t height,  size_t width,
					     bool loop = false);

    MMatrix<T>& shift(int deltaHeight, int deltaWidth, const T& fill=T());
    MMatrix<T>&  loop(int deltaHeight, int deltaWidth);

    MMatrix<T>& flip(bool bottomUp = true, bool leftRight = true);
    
    template <typename U,typename V> MMatrix<T>& operator()( U (*f)(const V&));
    template <typename U,typename V> MMatrix<T>& operator()( U (*f)(V) );
    template <typename U> MMatrix<T>& operator()(const U&);
    pair<T,T> meanAndStandardDeviation(void) const;
    T mean(void) const;
    T standardDeviation(void) const {return meanAndStandardDeviation().second;}
    T max(void) const;
    T min(void) const;
    T norm1(void) const;
    T norm2(void) const;
    T normInf(void) const;
    MMatrix<T>& oppose(void);
    MMatrix<T>& inverse(void);
    MMatrix<T>& sqr(void);
    MMatrix<T>& sqrt(void) {return (*this)( (double (*)(double)) &std::sqrt);}
    MMatrix<T>& abs(void);
		template<typename U,typename V>
    MMatrix<T>& mulAdd(const MMatrix<U>& m1, const MMatrix<V>& m2);
		MMatrix<T>& pow(long n);
		MMatrix<T>& pow(unsigned long n);
		MMatrix<T>& pow(int n)            {return pow(static_cast<long>(n));}
		MMatrix<T>& pow(unsigned int  n)  {return pow(static_cast<unsigned long>(n));}
		MMatrix<T>& pow(short n)          {return pow(static_cast<long>(n));}
		MMatrix<T>& pow(unsigned short n) {return pow(static_cast<unsigned long>(n));}
		MMatrix<T>& pow(char n)           {return pow(static_cast<long>(n));}
		MMatrix<T>& pow(unsigned char n)  {return pow(static_cast<unsigned long>(n));}
		MMatrix<T>& pow(double y);
    bool any(void) const;
    bool none(void) const {return !any();}
    bool every(void) const;
    template<typename U> T dotProduct(const MMatrix<U>& other) const;
    template<typename U> MMatrix<T> mulByVec(const MMatrix<U>& lineMMatrix) const;

  public:
    MMatrix<T>& writeToStream(ostream&) const;
    MMatrix<T>& readFromStream(istream&);
    friend ostream& operator<< <> (ostream&,const MMatrix<T>&);
    friend istream& operator>> <> (istream&,MMatrix<T>&);
    
  public:
    size_t getHeight()     const {return height;}
    size_t getWidth()      const {return width;}
    T**          getData()       const {return data;}
    size_t getDataLength() const {return height*width;}
    bool  getBinaryMode(void)    const {return binaryMode;}
    void  setBinaryMode(bool mode)     {binaryMode = mode;}

  public:
    static T DotProduct(const MMatrix<T>& m1, const MMatrix<T>& m2)
		       {return m1.dotProduct(m2);}

    static void FFT2D(MMatrix<T>* Output_real, MMatrix<T>* Output_imag,
		      const MMatrix<T>& Input_real, const MMatrix<T>& Input_imag);
    static void IFFT2D(MMatrix<T>* Output_real, MMatrix<T>* Output_imag,
		       const MMatrix<T>& Input_real, const MMatrix<T>& Input_imag);

    #ifdef USE_ALTIVEC
    static void FFT2D(MMatrix<T>* Output_real, MMatrix<T>* Output_imag,
		      const MMatrix<T>& Input_real, const MMatrix<T>& Input_imag,
		      const FFTSetup& setup);
    static void IFFT2D(MMatrix<T>* Output_real, MMatrix<T>* Output_imag,
		       const MMatrix<T>& Input_real, const MMatrix<T>& Input_imag,
		       const FFTSetup& setup);
    #endif

  private:
    size_t height;
    size_t width;
    T** data;
    bool binaryMode;

  private:
    T**  allocation(size_t height, size_t width, const T& fill= T());
    void clean(void);
    void memorySet(T* data,size_t size, const T& fill = T());
    void memoryCopy(T* dest, const T* src, size_t size);
    void memoryMove(T* dest, const T* src, size_t size);

    void copy(const MMatrix<T>& other);
    template<typename U> void copy(const MMatrix<U>& other);

    void copy(const MMatrix<T>& other, size_t fromRow, size_t fromCol,
		                      size_t height,  size_t width,
				      bool loop = false);
    template <typename U>
    void copy(const MMatrix<U>& other, size_t fromRow, size_t fromCol,
		                      size_t height,  size_t width,
				      bool loop = false);
  private:
    static T abs(const T&);
    static bool CheckFFTOk(const MMatrix<T>* Output_real, const MMatrix<T>* Output_imag,
                           const MMatrix<T>& Input_real , const MMatrix<T>& Input_imag );
    static void Four2(MMatrix<T>* fftr, MMatrix<T>* ffti,
		      MMatrix<T>* datar,MMatrix<T>* datai, int isign);
    static void Four1(MMatrix<T>* data, int isign);
    template<typename U> static T DotProduct(const T* data1, const U* data2, size_t size);
    static size_t LogTwoOfCeilingPowerOfTwo(size_t x);
};

template <typename T>
inline MMatrix<T> operator+(const MMatrix<T>& m1, const MMatrix<T>& m2) {MMatrix<T> m(m1); return m+=m2;}
template <typename T>
inline MMatrix<T> operator-(const MMatrix<T>& m1, const MMatrix<T>& m2) {MMatrix<T> m(m1); return m-=m2;}
template <typename T>
inline MMatrix<T> operator*(const MMatrix<T>& m1, const MMatrix<T>& m2) {MMatrix<T> m(m1); return m*=m2;}
template <typename T>
inline MMatrix<T> operator/(const MMatrix<T>& m1, const MMatrix<T>& m2) {MMatrix<T> m(m1); return m/=m2;}

template <typename T>
inline MMatrix<T> operator&(const MMatrix<T>& m1, const MMatrix<T>& m2) {MMatrix<T> m(m1); return m&=m2;}
template <typename T>
inline MMatrix<T> operator|(const MMatrix<T>& m1, const MMatrix<T>& m2) {MMatrix<T> m(m1); return m|=m2;}
template <typename T>
inline MMatrix<T> operator^(const MMatrix<T>& m1, const MMatrix<T>& m2) {MMatrix<T> m(m1); return m^=m2;}

template <typename T,typename U>
inline MMatrix<T> operator+(const MMatrix<T>& m1, U x) {MMatrix<T> m(m1); return m+=x;}
template <typename T,typename U>
inline MMatrix<T> operator+(U x, const MMatrix<T>& m1) {MMatrix<T> m(m1); return m+=x;}

template <typename T,typename U>
inline MMatrix<T> operator-(const MMatrix<T>& m1, U x) {MMatrix<T> m(m1); return m-=x;}
template <typename T,typename U>
inline MMatrix<T> operator-(U x, const MMatrix<T>& m1) {MMatrix<T> m(m1); return m-=x;}

template <typename T,typename U>
inline MMatrix<T> operator*(const MMatrix<T>& m1, U x) {MMatrix<T> m(m1); return m*=x;}
template <typename T,typename U>
inline MMatrix<T> operator*(U x, const MMatrix<T>& m1) {MMatrix<T> m(m1); return m*=x;}

template <typename T,typename U>
inline MMatrix<T> operator/(const MMatrix<T>& m1, U x) {MMatrix<T> m(m1); return m/=x;}
template <typename T,typename U>
inline MMatrix<T> operator/(U x, const MMatrix<T>& m1) {MMatrix<T> m(m1); return m/=x;}

template <typename T,typename U>
inline MMatrix<T> operator&(const MMatrix<T>& m1, U x) {MMatrix<T> m(m1); return m&=x;}
template <typename T,typename U>
inline MMatrix<T> operator&(U x, const MMatrix<T>& m1) {MMatrix<T> m(m1); return m&=x;}
template <typename T,typename U>
inline MMatrix<T> operator|(const MMatrix<T>& m1, U x) {MMatrix<T> m(m1); return m|=x;}
template <typename T,typename U>
inline MMatrix<T> operator|(U x, const MMatrix<T>& m1) {MMatrix<T> m(m1); return m|=x;}
template <typename T,typename U>
inline MMatrix<T> operator^(const MMatrix<T>& m1, U x) {MMatrix<T> m(m1); return m^=x;}
template <typename T,typename U>
inline MMatrix<T> operator^(U x, const MMatrix<T>& m1) {MMatrix<T> m(m1); return m^=x;}

template <typename T, typename U>
inline bool operator==(const MMatrix<T>& m1, const MMatrix<U>& m2);
template <typename T, typename U>
inline bool operator!=(const MMatrix<T>& m1, const MMatrix<U>& m2) {return !(m1==m2);}
template <typename T, typename U>
inline bool operator<(const MMatrix<T>& m1, const MMatrix<U>& m2);
template <typename T, typename U>
inline bool operator<=(const MMatrix<T>& m1, const MMatrix<U>& m2);
template <typename T, typename U>
inline bool operator>(const MMatrix<T>& m1, const MMatrix<U>& m2);
template <typename T, typename U>
inline bool operator>=(const MMatrix<T>& m1, const MMatrix<U>& m2);

template<typename T> inline ostream& operator<<(ostream&,const MMatrix<T>&);
template<typename T> inline istream& operator>>(istream& stream, MMatrix<T>& m)
                     {m.readFromStream(stream);return stream;}

#endif
