
#include <cstdio>
#include <cstdlib>
#include <iomanip>
#include <cmath>
#include <iostream>
#include <string>
#include <vector>
#include <algorithm>
#include <plot.h>

#include "m_matrix2.h"
#include "maths_util.h"
#include "ToricExtremal.h"
#include "plotting.h"
#include "MatrixFile.h"
#include "Polygon.h"
#include "xyzFileSave.h"
using namespace std;

enum mm_enum {mm_defect = 1, mm_bach, mm_riemnorm, mm_weyl, mm_ricci, mm_gauss, mm_numMinMaxs};

int main(int argc, char *argv[])
{	
	
	//Key variables to be saved and loaded
	int k;
	int NV;
	Polygon pg;
	MMatrix<double> B; //metric matrix (size is set later)
	
	//parameters loaded from params file
	string fileName;
	char loadingMode;
	string pgInput;
	int graphnumber = 0;
	int plotType = 0;
	char inputPlotOutput;
	int saveInterval = 1;
	bool storeL2Norm = false;
	double xyzElimCloseness = 0.1;
	
	
	///////////////////////
	//Load Parameter File//
	///////////////////////
	string paramFileName = "params.txt";
	ifstream param(paramFileName.c_str()); 
	if(param.is_open())
	{
		param >> fileName;
		skipline(param);
		param >> loadingMode;
		skipline(param);
		param >> k;
		skipline(param);
		getline(param, pgInput);
		param >> graphnumber;
		skipline(param);
		param >> plotType;
		skipline(param);
		param >> inputPlotOutput;
		skipline(param);
		param >> saveInterval;
		skipline(param);
		param >> storeL2Norm;
		skipline(param);
		param >> xyzElimCloseness;
		skipline(param);
	}
	else
	{
		cout << "Could not open parameter file: " << paramFileName << endl;
		return 1;
	}
	param.close();
	
	
	
	///////////////////////////////////////
	//Initialise file objects and Polygon//
	///////////////////////////////////////
	
	//main results file object for loading or saving
	MatrixFile resultsFile(fileName);
	resultsFile.setMatrixLabel("Matrix B");
	
	bool newCalc = false; //whether we are starting a new calculation or using loaded results
	bool viewOnly = !(loadingMode == 'n' || loadingMode == 'l'); 
		//whether we are just viewing results or carrying on iteration
	
	if (loadingMode == 'n')
	{ //starting new calculation
		newCalc = true;
		
		bool parseResult = pg.parse(pgInput);
		if (!parseResult)
		{
			cout << "Invalid polygon string given in parameter file" << endl;
			return 1;
		}
		//DEBUG - cout << pg.numVerts() << endl;
		//DEBUG - cout <<  pg.toEdgeString();
		cout << "Starting new calculation in file " << fileName << endl;
		
		resultsFile.setMatrixNum(0);
		resultsFile.save("Toric Extremal Results");
		resultsFile.save("k", k);
		resultsFile.save("Polygon", pg.toEdgeString());
		
		B.resize(k+1,k+1,0.); //makes B a k+1 by k+1 matrix filled with 0.0
	}
	else
	{ //loading previous results
		newCalc = false;
		string polyString = resultsFile.loadLastString("Polygon");
		//DEBUG - cout << polyString << endl;
		bool parseResult = pg.parse(polyString);
		if (!parseResult)
		{
			cout << "Failed to load valid polygon string from file" << endl;
			return 1;
		}
		resultsFile.loadLastMatrix(B); //LOADING METRIC HERE
		resultsFile.incrementNum();
		k = B.getHeight() - 1;
		if (B.getHeight() != B.getWidth() || k < 3)
		{
			cout << "Loaded Matrix B invalid" << endl;
			return 1;
		}
		
		cout << "Loaded results from file " << fileName << endl;
		//DEBUG - cout.precision(21);
		//DEBUG - cout << B[0][0] << "     " << B[7][1] << endl;
		//DEBUG - show(B);
		//DEBUG - resultsFile.save(B);
		
	}
	NV = pg.numVerts();
	cout << "k: " << k << endl;
	cout << "Polygon string: " << pg.toEdgeString() << endl;
	
	double refiner1;//coefficients to choose for refining process
	int numIterations = 100000; //total number of iterations to complete
	const double stopconstant2 = .0008;
	double kreal = double(k);
	int refnumber = 0;
	int runnumber = 0;
	double newdel = 0;
	int ltrunc1;
	int ltrunc2;
	
	
	/////////////////////////////////
	//Libplot plotter initialisation/
	/////////////////////////////////
	cout << "graph number: " << graphnumber << endl;
	cout << "plot type: " << plotType << endl;
	bool libplotPlotting;
	if (graphnumber == 0 || plotType >= 10)
	{
		libplotPlotting = false;
	}
	else
	{
		libplotPlotting = true;
	}
	
	CPlotter libPlotPlotter;
	if (libplotPlotting)
	{
		ostringstream pictFilePathS;
		pictFilePathS << fileName << "_" << graphnumber;
			
		libPlotPlotter.open(inputPlotOutput, pictFilePathS.str());
		libPlotPlotter.printDetails();
		
		if (graphnumber == 22)
		{
			pl_fspace (-.05, -0.05, 1.05, 1.05);
		}
		else
		{
			pl_space (-1, -1, k + 1, k + 1);
		}
	}
	cout << " OK ";
	//code to cut off all plotting outside a cut off circle - not in use
	Circle plotCutOffCircle;
	const bool plotCutOff = false;
	if (graphnumber != 0 && plotCutOff)
	{
		plotCutOffCircle.set(pg.x1()[0], pg.x2()[0], ((double) k)/10.);
		//DEBUG - cout << "plotting cut off " << plotCutOffCircle.toString() << endl;
	}
	
	
	////////////////////////////
	//Setup details for xyzFile/
	////////////////////////////
	// file for xd3d (or matlab) results
	string xyzFileName;
	if (plotType >= 10)
	{
		ostringstream xyzNameStream;
		xyzNameStream << fileName << "_" << graphnumber << "_" << xyzElimCloseness;
		if (plotType == 11)
		{
			xyzNameStream << ".csv";
		}
		else
		{
			xyzNameStream << ".xyz";
		}
		xyzFileName = xyzNameStream.str();
		cout << "xyz output file: " << xyzFileName << endl;
	}
	
	////////////////////////////
	//Setup details for l2normFile/
	////////////////////////////
	//file to save l2norms of scalar curvature defects
	string l2normFileName;
	const int l2normPrecision = 15;
	if (storeL2Norm && !viewOnly)
	{
		ostringstream l2normNameStream;
		l2normNameStream << fileName << "_scalarcurvaturedefects";
		l2normFileName = l2normNameStream.str();
		cout << "l2norm output file: " << l2normFileName << endl;
		ofstream l2normFile;
		l2normFile.open(l2normFileName.c_str(), ios_base::out | ios_base::trunc);
		if (!l2normFile.is_open())
		{
			cout << "failed to open file to store l2norms" << endl;
			return 1;
		}
		l2normFile.close();
	}
	
	
	//////////////////////////
	//Further Initialisation//
	//////////////////////////
	int counter = 0; //counter for the output -1
	double Area = 0.;
	double trianglearea;
	int boundarynumber = 0;
	double polymom1 = 0.;
	double polymom2 = 0.;
	double boundmom1 = 0.;
	double boundmom2 = 0.;
	double I11 = 0.;
	double I12 = 0.;
	double I22 = 0.;

	for ( int i = 1;i <= NV;i++)
	{
		double Pir = double(pg.x1()[i]);
		double Pjr = double(pg.x1()[i-1]);
		double Qir = double(pg.x2()[i]);
		double Qjr = double(pg.x2()[i-1]);
		trianglearea = .5 * pg.eLen()[i-1] * (Pjr * double(pg.x2d()[i-1]) 
						- Qjr * double(pg.x1d()[i-1]));
		Area = Area + trianglearea;
		polymom1 += trianglearea * ((Pir / 3.) + (Pjr / 3.) - (kreal / 2.));
		polymom2 += trianglearea * ((Qir / 3.) + (Qjr / 3.) - (kreal / 2.));
		I11 = I11 + trianglearea * (Pir * Pir + Pir * Pjr + Pjr * Pjr) / 6.;
		I22 = I22 + trianglearea * (Qir * Qir + Qir * Qjr + Qjr * Qjr) / 6.;
		I11 = I11 + trianglearea * (((kreal / 2.) 
				- 2.*(Pir + Pjr) / 3.) * (kreal / 2.));
		I22 = I22 + trianglearea * (((kreal / 2.) 
				- 2.*(Qir + Qjr) / 3.) * (kreal / 2.));
		I12 = I12 + trianglearea * (Pir * Qir + Pjr * Qjr 
									+ (Pir * Qjr + Pjr * Qir) / 2.) / 6.;
		I12 = I12 + trianglearea * (((kreal / 2.) 
				- (Pir + Pjr + Qir + Qjr) / 3.) * (kreal / 2.));
		boundarynumber = boundarynumber + pg.eLen()[i-1];
		boundmom1 = boundmom1 + double(pg.eLen()[i-1]) * (Pir + Pjr - kreal) / 2.;
		boundmom2 = boundmom2 + double(pg.eLen()[i-1]) * (Qir + Qjr - kreal) / 2.;

	}

	double boundcm1 = boundmom1 / double(boundarynumber);
	double boundcm2 = boundmom2 / double(boundarynumber);
	double momdefect1 = boundcm1 - polymom1 / Area;
	double momdefect2 = boundcm2 - polymom2 / Area;
	I11 = I11 - polymom1 * polymom1 / Area;
	I22 = I22 - polymom2 * polymom2 / Area;
	I12 = I12 - polymom1 * polymom2 / Area;

	
	double mu1 = I22 * momdefect1 - I12 * momdefect2;
	double mu2 = -I12 * momdefect1 + I11 * momdefect2;
	double detI = I11 * I22 - I12 * I12;
	mu1 = mu1 * double(boundarynumber) / detI;
	mu2 = mu2 * double(boundarynumber) / detI;
	double graphver1 = kreal / (1 + 4.*log(20.*kreal));
	double graphver2 = 4.*log(20.*kreal) * graphver1;
	double invasc = Area / double(boundarynumber);
	double asc = 1. / invasc;
	double tau1 = polymom1 / Area + kreal / 2.;
	double tau2 = polymom2 / Area + kreal / 2.;
	MMatrix<bool> chi(k+1,k+1);
	for (int p = 0;p <= k;p++)
	{
		for (int q = 0;q <= k;q++)
		{
			chi[p][q] = pg.contains((double) p, (double) q);
		}
	}//this is an indicator function for the integral points in the polygon

	int polynumber = 0;
	double Zmom1 = 0.;
	double Zmom2 = 0.;
	double ZI11 = 0.;
	double ZI12 = 0.;
	double ZI22 = 0.;
	for (int i = 0;i <= k;i++)
	{
		for (int j = 0;j <= k;j++)
		{
			if (chi[i][j])
			{
				polynumber = polynumber + 1;
				Zmom1 = Zmom1 + (double(i) - kreal / 2.);
				Zmom2 = Zmom2 + (double(j) - kreal / 2.);
				ZI11 = ZI11 + (double(i) - kreal / 2.) * (double(i) - kreal / 2.);
				ZI12 = ZI12 + (double(i) - kreal / 2.) * (double(j) - kreal / 2.);
				ZI22 = ZI22 + (double(j) - kreal / 2.) * (double(j) - kreal / 2.);
			}
		}
	}
	ZI11 = ZI11 - Zmom1 * Zmom1 / double(polynumber);
	ZI12 = ZI12 - Zmom1 * Zmom2 / double(polynumber);
	ZI22 = ZI22 - Zmom2 * Zmom2 / double(polynumber);
	double detZI = ZI11 * ZI22 - ZI12 * ZI12;

	refiner1 = .7 * (double(boundarynumber) / double(polynumber));
	MMatrix<double> powers(k+1,k+1, 0.);//array to hold monomials at each point
	MMatrix<double> delt(k+1,k+1, 0.);//array of refining corrections
	MMatrix<double> rdelt(k+1,k+1, 0.);//retrospective corrections

	
	//If not already loaded from file, initialise B to sensible values
	if (newCalc)
	{
		for (int i = 0;i <= k;i++)
		{
			for (int j = 0;j <= k;j++)
			{
				if (chi[i][j])
				{
					for (int bet = 0; bet <= NV - 1; bet++)
					{
						double lamb 
							= double(pg.x1d()[bet]) 
									* (double(j) - double(pg.x2()[bet]))
							- double(pg.x2d()[bet]) 
									* (double(i) - double(pg.x1()[bet]));
						lamb = lamb / kreal;
						B[i][j] = B[i][j] - XlogX(lamb);
					}
					B[i][j] = exp(.75 * kreal * B[i][j]);
				}
			}
		}
	}


	double zmax = 0.;
	double x1bad = 0.;
	double x2bad = 0.;
	double sum = 0.;
	double sum2 = 0.;
	double sum3 = 0.;
	double sum4 = 0.;
	double sum5 = 0.;
	double sum6 = 0.;
	double scaldefmax = 0.;
	double scaldefmin = 0.;
	double scaldef2max = 0.;
	double scaldef2min = 0.;
	double sum3previous = 0.;


		//the array of integrals for the balancing algorithm
	MMatrix<double> PINTEGRAL(k+1,k+1);
		//the array of scalar curvature integrals
	MMatrix<double> SINTEGRAL(k+1,k+1);

	double lam[NV-1];//these are the values of the affine linear functions
	int alp = 0;//label of chart we are in
	int alpnew;
	int gam = NV - 1;
	double mindist;//criterion for choice of chart
	double x1;//coordinates of point in polygon
	double x2;
	double z1;//coordinates in our chart
	double z2;
	double w1;
	double w2;
	double DD[5][5];
	double ratal;//ratios of z-variables and lambda variables
	double ratgam;
	double acomp;//components of  Hessian matrix
	double bcomp;
	double dcomp;
	double Aab;
	double Bab;
	double Jac;//gives the Jacobian for change of variables in integrals
	bool indic;
	double mono;
	double monohold;
	int count;
	double badness;
	int chartmove;
	int maxwidth = 0;
	int width;
	int height;
	for (int i = 1;i < NV;i++)
	{
		for (int j = 0;j < NV;j++)
		{
			width = (pg.x2()[j] - pg.x2()[i]) * pg.x1d()[i-1] 
					- (pg.x1()[j] - pg.x1()[i]) * pg.x2d()[i-1];
			height = (pg.x2()[j] - pg.x2()[i]) * pg.x1d()[i] 
					- (pg.x1()[j] - pg.x1()[i]) * pg.x2d()[i];
			//debug - cout << "height: " << height << "width: " << width;
			if (width > maxwidth)
			{
				maxwidth = width;
			}
			if (height > maxwidth)
			{
				maxwidth = height;
			}
		}
	}

	for (int j = 1;j < NV;j++)
	{
		width = (pg.x2()[j] - pg.x2()[0]) * pg.x1d()[NV-1] 
				- (pg.x1()[j] - pg.x1()[0]) * pg.x2d()[NV-1];
		height = (pg.x2()[j] - pg.x2()[0]) * pg.x1d()[0] 
				- (pg.x1()[j] - pg.x1()[0]) * pg.x2d()[0];
		if (width > maxwidth)
		{
			maxwidth = width;
		}
		if (height > maxwidth)
		{
			maxwidth = height;
		}
	}

	int Trunc = int(2.95 * pow(double(maxwidth), .57));
	int intnumber = int(27. * pow(kreal, .55));
	
	cout << endl;
	cout << "Maxwidth: " << maxwidth << endl;
	cout << "intnumber: " << intnumber << endl;
	cout << "Truncation: " << Trunc << endl;
	
	if (viewOnly)
	{
		numIterations = resultsFile.getMatrixNum() + 1; //one more iteration
	}
	else
	{
		cout << "Press enter to begin iteration..." << endl;
		//cin.ignore(100, '\n'); //clean up new line character left in cin buffer by earlier uses of <<
		//skipline(cin);
		cin.get();
	}
	
	
	//plot axes for libplot graph of l2norms of scalar curvature defect
	if (libplotPlotting)
	{
		pl_erase();
		if (graphnumber == -1)
		{
			pl_colorname("blue");
			pl_fline(0., 0., kreal, 0.);
			pl_fline(0., 0., 0., kreal);
			pl_fontname("Helvetica");
			pl_fontsize(-1);
			pl_pencolorname("Black");
			pl_fmove(0., graphver1 + graphver2);
			pl_label("1");
			pl_fmove(0., graphver1*(-2.) + graphver2);
			pl_label("-2");
			pl_fmove(0., graphver1*(-5.) + graphver2);
			pl_label("-5");
			pl_fmove(2.5, 0.);
			pl_label("100");
		}
		else
		{
			pl_pencolorname("green");
			drawPolygon(pg);
		}
	}
	
	double intstep = 1 / (double(intnumber));

	double powlist1[2*Trunc+9];
	double powlist2[2*Trunc+9];
	double simpleterms[2*Trunc+1][5];
	for (int i = -Trunc;i <= Trunc;i++)
	{
		for (int a = 0;a <= 4;a++)
		{
			simpleterms[i+Trunc][a] = smallprod(i, a);
		}
	}
	
	///////////////////////////
	//START OF ITERATION LOOP//
	///////////////////////////
	for (int itnumber = resultsFile.getMatrixNum(); itnumber < numIterations; itnumber++)
	{
		//libplot background for points
		bool plotResults = (itnumber % 10 == 0 || viewOnly);
		if (plotResults && libplotPlotting)
		{
			if ((graphnumber > 20 && graphnumber < 30) | plotType == 2)
			{
				pl_flinewidth(0.03);
				pl_pencolorname("black");
				pl_bgcolorname("white");
			}
			else
			{
				pl_pencolorname("green");
				pl_bgcolorname("black");
			}
			pl_erase();
			if (graphnumber == 22)
			{
				pl_flinewidth(0.005);
				
				//draw a unit square
				pl_line(0,0,1,0);
				pl_line(1,0,1,1);
				pl_line(1,1,0,1);
				pl_line(0,1,0,0);
			}
			else
			{
				drawPolygon(pg);
			}
			if (plotCutOff)
			{
				pl_pencolorname("BLUE");
				draw(plotCutOffCircle);
			}
			
			pl_pencolorname("black"); //default colour for further drawing
		}
		
		//containers for storage of results for plotting and display
		vector<Vector3> plottingResults;
		vector<MinMax> minMaxes(mm_numMinMaxs);

		runnumber = runnumber + 1;
		if (newdel == 1)
		{
			for (int a = 0;a <= k;a++)
			{
				for (int b = 0;b <= k;b++)
				{
					if (chi[a][b])
					{
						delt[a][b] = rdelt[a][b] + refiner1 * SINTEGRAL[a][b];//this is the new approach
						runnumber = 0;
					}
				}
			}
			refnumber = refnumber + 1;
		}


		int basep = 0;
		int baseq = 0;
		int basei = 0;
		int basej = 0;
		sum3previous = sum3;
		sum = 0.;
		sum2 = 0.;
		sum3 = 0.;
		sum4 = 0.;
		sum5 = 0.;
		sum6 = 0.;
		scaldefmax = 0.;
		scaldefmin = 0.;
		scaldef2max = 0.;
		scaldef2min = 0.;
		for (int a = 0;a <= k;a++)
		{
			for (int b = 0;b <= k;b++)
			{
				PINTEGRAL[a][b] = 0.;
				SINTEGRAL[a][b] = 0.;
			}
		}

		zmax = 0.;

		int sign = -1;
		alp = 0;
		for (int f = 0;f <= intnumber;f++)
		{ //START F-LOOP
			sign = -sign;
			for (int g = 0;g <= intnumber;g++)
			{		//START G-LOOP


				x1 = kreal * double(f) * intstep;
				if (sign == 1)
				{
					x2 = kreal * double(g) * intstep;
				}
				if (sign == -1)
				{
					x2 = kreal * (1. - double(g) * intstep);
				}
				indic = true;//this is false if outside polygon
				for (int bet = 0;bet <= NV - 1;bet++)
				{
					lam[bet] = double(pg.x1d()[bet]) * (x2 - double(pg.x2()[bet])) 
							- double(pg.x2d()[bet]) * (x1 - double(pg.x1()[bet]));
					lam[bet] = lam[bet] / kreal;
					if (lam[bet] < -.000001)
					{
						indic = false;
					}
					if (lam[bet] < 0.)
					{
						lam[bet] == 0.;
					}
				}

				if (indic)//START OF MAIN LOOP:we only calculate if inside polygon
				{
					
					
					alpnew = alp;
					badness = 2.;
					count = 0;
					do
					{				//start of loop to find a good chart
						alp = alpnew;
						ratal = 1.;//setting all these to 0 or 1 before the sum or product below
						ratgam = 1.;
						acomp = 0.;
						bcomp = 0.;
						dcomp = 0.;
						gam = cyclic(alp, NV, -1);

						for (int bet = 0;bet <= NV - 1;bet++)
						{
							if (bet != alp && bet != gam)
							{
								ratal *= lowpow(lam[bet], 
											pg.x1d()[gam] * pg.x2d()[bet] 
											- pg.x1d()[bet] * pg.x2d()[gam]);
								ratgam *= lowpow(lam[bet], 
											pg.x1d()[bet] * pg.x2d()[alp] 
											- pg.x1d()[alp] * pg.x2d()[bet]);
								Aab = double(pg.x1d()[gam] * pg.x2d()[bet] 
										- pg.x1d()[bet] * pg.x2d()[gam]);
								Bab = double(pg.x1d()[bet] * pg.x2d()[alp] 
										- pg.x1d()[alp] * pg.x2d()[bet]);
								acomp = acomp + (1 / lam[bet]) * Aab * Aab;
								bcomp = bcomp + (1 / lam[bet]) * Aab * Bab;
								dcomp = dcomp + (1 / lam[bet]) * Bab * Bab;

							}
						}

						z2 = lam[alp] * ratal;
						z1 = lam[gam] * ratgam;

						badness = z2;
						chartmove = -1;
						if (z1 >= z2)
						{
							chartmove = 1;
							badness = z1;
						}
						if (badness > 1.01)
						{
							alpnew = cyclic(alp, NV, chartmove);
							count = count + 1;
						}



					}
					while (badness > 1.01 && count < NV);//end of "while badness" loop. Now we have a choice of chart, label "alp"
					if (z1 > zmax)
					{
						zmax = z1;
					}
					if (z2 > zmax)
					{
						zmax = z2;
					}

					Jac = 1 + lam[alp] * acomp + lam[gam] * dcomp 
							+ lam[alp] * lam[gam] * (acomp * dcomp - bcomp * bcomp); // CHANGED THIS
					Jac = Jac * lam[alp] * lam[gam] * ratal * ratal 
							* ratgam * ratgam;

					w1 = z1 * z1;//now we start doing all calculations at this point
					w2 = z2 * z2;

					basei = (baseq - pg.x2()[alp]) * pg.x1d()[gam] 
							- (basep - pg.x1()[alp]) * pg.x2d()[gam];
					basej = (baseq - pg.x2()[alp]) * pg.x1d()[alp] 
							- (basep - pg.x1()[alp]) * pg.x2d()[alp];

					int ppp;
					int qqq;


					if (w1 < .1)
					{
						basei = 0;
					}
					if (w2 < .1)
					{
						basej = 0;
					}

					ltrunc1 = basei;
					if (ltrunc1 > Trunc)
					{
						ltrunc1 = Trunc;
					}
					ltrunc2 = basej;
					if (ltrunc2 > Trunc)
					{
						ltrunc2 = Trunc;
					}



					for (int a = 0;a <= 4;a++)
					{
						for (int b = 0;b <= 4;b++)
						{
							DD[a][b] = 0.;
						}
					}
					int p = pg.x1()[alp] + basei * pg.x1d()[alp] 
							- basej * pg.x1d()[gam];
					int q = pg.x2()[alp] + basei * pg.x2d()[alp] 
							- basej * pg.x2d()[gam];

					for (int i = 0;i <= 2 * Trunc + 8; i++)
					{
						powlist1[i] = 0.;
						powlist2[i] = 0.;
					}
					powlist1[Trunc+4] = 1.;
					powlist2[Trunc+4] = 1.;
					for (int i = 1;i <= Trunc + 4;i++)
					{
						powlist1[Trunc+4+i] = powlist1[Trunc+3+i] * w1;
						powlist2[Trunc+4+i] = powlist2[Trunc+3+i] * w2;
						if (i <= ltrunc1 + 4)
						{
							powlist1[Trunc+4-i] = 1. / powlist1[Trunc+4+i];
						}
						if (i <= ltrunc2 + 4)
						{
							powlist2[Trunc+4-i] = 1. / powlist2[Trunc+4+i];
						}
					}

					for (int i = -ltrunc1;i <= Trunc;i++)
					{
						for (int j = -ltrunc2;j <= Trunc;j++)
						{


							int r = p + i * pg.x1d()[alp] - j * pg.x1d()[gam];
							int s = q + i * pg.x2d()[alp] - j * pg.x2d()[gam];

							if (inSquare(r, s, k))
							{
								if (chi[r][s])
								{
									powers[r][s] = powlist1[Trunc+4+i] 
													* powlist2[Trunc+4+j];
									for (int a = 0;a <= 4;a++)
									{
										for (int b = 0;b <= 4 - a;b++)
										{

											double term = simpleterms[i+Trunc][a] 
														* simpleterms[j+Trunc][b];
											if (term * term > .01)
											{
												double sumterm = term * B[r][s] 
														* powlist1[Trunc+4+i-a] 
														* powlist2[Trunc+4+j-b];
												DD[a][b] = DD[a][b] + sumterm;
											}	//end if
										}	//end b
									}	//end a
								}//end if
							}//end if
						}//endj
					}//endi


					double F = DD[0][0];
					double p10 = DD[1][0] / F;//pab denotes derivatives of phi=logF,a derivs in w1 and b in w2;
					double p01 = DD[0][1] / F;
					double p20 = DD[2][0] / F - p10 * p10;
					double p11 = DD[1][1] / F - p10 * p01;
					double p02 = DD[0][2] / F - p01 * p01;
					double p30 = 3.*p10 * p20 + p10 * p10 * p10;
					double p21 = 2.*p10 * p11 + p01 * p20 + p10 * p10 * p01;
					double p12 = 2.*p01 * p11 + p10 * p02 + p01 * p01 * p10;
					double p03 = 3.*p01 * p02 + p01 * p01 * p01;
					p30 = (DD[3][0] / F) - p30;
					p21 = (DD[2][1] / F) - p21;
					p12 = (DD[1][2] / F) - p12;
					p03 = (DD[0][3] / F) - p03;

					double p40 = 4.*p10 * p30 + 3.*p20 * p20 + 6.*p20 * p10 * p10 + p10 * p10 * p10 * p10;
					double p04 = 4.*p01 * p03 + 3.*p02 * p02 + 6.*p02 * p01 * p01 + p01 * p01 * p01 * p01;
					double p31 = 3.*p10 * p21 + p01 * p30 + 3.*p11 * p20 + 3.*p20 * p10 * p01 + 3.*p11 * p10 * p10 + p01 * p10 * p10 * p10;
					double p13 = 3.*p01 * p12 + p10 * p03 + 3.*p11 * p02 + 3.*p02 * p01 * p10 + 3.*p11 * p01 * p01 + p10 * p01 * p01 * p01;
					double p22 = 2.*p10 * p12 + 2.*p01 * p21 + p20 * p02 + 2.*p11 * p11 + p20 * p01 * p01 + p02 * p10 * p10;
					p22 = p22 + 4.*p11 * p10 * p01 + p10 * p10 * p01 * p01;
					p40 = (DD[4][0] / F) - p40;
					p31 = (DD[3][1] / F) - p31;
					p22 = (DD[2][2] / F) - p22;
					p13 = (DD[1][3] / F) - p13;
					p04 = (DD[0][4] / F) - p04;
					p10 = asc * p10;//normalise scale
					p01 = asc * p01;
					p20 = asc * p20;
					p11 = asc * p11;
					p02 = asc * p02;
					p30 = asc * p30;
					p21 = asc * p21;
					p12 = asc * p12;
					p03 = asc * p03;
					p40 = asc * p40;
					p31 = asc * p31;
					p22 = asc * p22;
					p13 = asc * p13;
					p04 = asc * p04;
					double hxx = p10 + w1 * p20;//h is the metric tensor; x and y are other names for  z1,z2
					double hyy = p01 + w2 * p02;
					double hxy = z1 * z2 * p11;
					double det = (hxx * hyy - hxy * hxy);

					double ixx = hyy / det; //i is the inverse (upper indices)
					double ixy = -hxy / det;
					double iyy = hxx / det;

					double cPx = z1 * (2.*p20 + w1 * p30);//these are the third deriv. terms
					double cRy = z2 * (2.*p02 + w2 * p03);//P,Q,R stand for xx, xy,yy
					double cQx = z2 * (p11 + w1 * p21);
					double cQy = z1 * (p11 + w2 * p12);
					double cRx = w2 * z1 * p12;
					double cPy = w1 * z2 * p21;

					double phiPP = 2.*p20 + 4.*w1 * p30 + w1 * w1 * p40;//these are the fourth deriv. terms
					double phiRR = 2.*p02 + 4.*w2 * p03 + w2 * w2 * p04;
					double phiPQ = z2 * z1 * (2.*p21 + w1 * p31);
					double phiQR = z1 * z2 * (2.*p12 + w2 * p13);
					double phiPR = w1 * w2 * p22;
					double phiQQ = p11 + w1 * p21 + w2 * p12 + w1 * w2 * p22;

					double BPP = cPx * cPx * ixx + (cPy * cPx + cPx * cPy) * ixy + cPy * cPy * iyy;//this computes quadratic in third derivs.
					double BRR = cRx * cRx * ixx + (cRy * cRx + cRx * cRy) * ixy + cRy * cRy * iyy;
					double BPQ = cPx * cQx * ixx + (cPx * cQy + cQx * cPy) * ixy + cPy * cQy * iyy;
					double BQR = cQx * cRx * ixx + (cQx * cRy + cRx * cQy) * ixy + cQy * cRy * iyy;
					double BPR = cPx * cRx * ixx + (cPx * cRy + cRx * cPy) * ixy + cPy * cRy * iyy;
					double BQQ = cQx * cQx * ixx + (cQx * cQy + cQy * cQx) * ixy + cQy * cQy * iyy;

					BPP = -(phiPP - BPP);//invasc*(phiPP-BPP)etc;
					BRR = -(phiRR - BRR);//(phiRR-BRR);
					BPQ = -(phiPQ - BPQ);//(phiPQ-BPQ);
					BQR = -(phiQR - BQR);//(phiQR-BQR);
					BPR = -(phiPR - BPR);//(phiPR-BPR);
					BQQ = -(phiQQ - BQQ);//(phiQQ-BQQ);//now BPQ etc are components of curvature tensor

					double ricxx = (BPP * ixx + 2.*BPQ * ixy + BQQ * iyy);//contract for Ricci
					double ricyy = (BRR * iyy + 2.*BQR * ixy + BQQ * ixx);
					double ricxy = (BPQ * ixx + BPR * ixy + BQQ * ixy + BQR * iyy);

					double scal = ricxx * ixx + 2.*ricxy * ixy + ricyy * iyy;
					double detricci = ricxx * ricyy - ricxy * ricxy;
					double tfricsize = sqrt(scal * scal - 4.*detricci / det); //absolute value of Ricci eigenvalue difference
					double gauss = (BPR - BQQ) / (4.*det); // -Gauss curvature in horizontal plane, up to a factor.
					double beta = gauss - scal / 8.;
					double gamma = 2.*det * beta;
					double tmu1 =  mu1 * double(pg.x1d()[alp]) + mu2 * double(pg.x2d()[alp]);
					double tmu2 = -(mu1 * double(pg.x1d()[gam]) + mu2 * double(pg.x2d()[gam]));
					double bachxx1 = p10 + 3.*w1 * p20 + w1 * w1 * p30;
					double bachxy1 = z1 * z2 * (p11 + w1 * p21);
					double bachyy1 = w1 * (p11 + w2 * p12);
					double bachxx2 = w2 * (p11 + w1 * p21);
					double bachxy2 = z1 * z2 * (p11 + w2 * p12);
					double bachyy2 = p01 + 3.*w2 * p02 + w2 * w2 * p03;
					double bachxx = invasc * invasc * (tmu1 * bachxx1 + tmu2 * bachxx2);
					double bachxy = invasc * invasc * (tmu1 * bachxy1 + tmu2 * bachxy2);
					double bachyy = invasc * invasc * (tmu1 * bachyy1 + tmu2 * bachyy2);
					double W[5];
					W[0] = BPP - ricxx * hxx - beta * hxx * hxx;
					W[1] = BPQ - .5 * (ricxx * hxy + ricxy * hxx) - beta * hxx * hxy;
					W[2] = BPR - .5 * (ricxx * hyy + ricyy * hxx) - beta * hxx * hyy - gamma;
					W[3] = BQR - .5 * (ricyy * hxy + ricxy * hyy) - beta * hyy * hxy;
					W[4] = BRR - ricyy * hyy - beta * hyy * hyy;

					double inv[2][2];
					double  S[2][2][2][2];
					double Riem[2][2][2][2];
					inv[0][0] = ixx;
					inv[0][1] = ixy;
					inv[1][0] = ixy;
					inv[1][1] = iyy;



					for (int i = 0;i <= 1;i++)
					{
						for (int j = 0;j <= 1;j++)
						{
							for (int k = 0;k <= 1;k++)
							{
								for (int l = 0;l <= 1;l++)
								{
									S[i][j][k][l] = W[i+j+k+l];
									if (i + j + k + l == 0)
									{
										Riem[i][j][k][l] = BPP;
									}
									if (i + j + k + l == 4)
									{
										Riem[i][j][k][l] = BRR;
									}
									if (i + j + k + l == 1)
									{
										Riem[i][j][k][l] = BPQ;
									}
									if (i + j + k + l == 3)
									{
										Riem[i][j][k][l] = BQR;
									}
									if (i + j + k + l == 2)
									{
										Riem[i][j][k][l] = BPR;
										if (i + j == 1)
										{
											Riem[i][j][k][l] = BQQ;
										}
									}
								}
							}
						}
					}

					double weyl = 0.;
					double Riemnorm = 0.;
					for (int i = 0;i <= 1;i++)
					{
						for (int j = 0;j <= 1;j++)
						{
							for (int k = 0;k <= 1;k++)
							{
								for (int l = 0;l <= 1;l++)
								{
									for (int a = 0;a <= 1;a++)
									{
										for (int b = 0;b <= 1;b++)
										{
											for (int c = 0;c <= 1;c++)
											{
												for (int d = 0;d <= 1;d++)
												{
													weyl += S[i][j][k][l] * S[a][b][c][d] * inv[i][a] * inv[j][b] * inv[k][c] * inv[l][d];
													Riemnorm += Riem[i][j][k][l] * Riem[a][b][c][d] * inv[i][a] * inv[j][b] * inv[k][c] * inv[l][d];
												}
											}
										}
									}
								}
							}
						}
					}
					//now we have finished all curvature calculations
					weyl = sqrt(weyl);
					Riemnorm = sqrt(Riemnorm);// norm of curvature tensor


					double shift1 = invasc * w1 * p10 + double(basei);
					double shift2 = invasc * w2 * p01 + double(basej);
					double xpt1 = double(pg.x1()[alp]) + shift1 * double(pg.x1d()[alp]) - shift2 * double(pg.x1d()[gam]);
					double xpt2 = double(pg.x2()[alp]) + shift1 * double(pg.x2d()[alp]) - shift2 * double(pg.x2d()[gam]);
					//these are the points in the hexagon which we map to under the Legendre transform.

					double scalexact = 1. + invasc * (mu1 * (xpt1 - tau1) + mu2 * (xpt2 - tau2));
					bachxx = 2.*bachxx + scalexact * ricxx;//before no invasc here
					bachxy = 2.*bachxy + scalexact * ricxy;
					bachyy = 2.*bachyy + scalexact * ricyy;//there is a tricky factor to get right here!!
					double bachtrace = bachxx * ixx + 2.*bachxy * ixy + bachyy * iyy;
					double detbach = bachxx * bachyy - bachxy * bachxy;
					double bachsize = sqrt(bachtrace * bachtrace - 4.*detbach / det);//norm of Bach tensor
					basei = int(shift1);
					basej = int(shift2);
					if (shift1 < .5)
					{
						basei = 0;
					}
					if (shift2 < .5)
					{
						basej = 0;
					}
					basep = pg.x1()[alp];
					baseq = pg.x2()[alp];
					int basepp = pg.x1()[alp] + basei * pg.x1d()[alp] - basej * pg.x1d()[gam];
					int baseqq = pg.x2()[alp] + basei * pg.x2d()[alp] - basej * pg.x2d()[gam];
					if (inSquare(basepp, baseqq, k) && chi[basepp][baseqq])
					{
						basep = basepp;
						baseq = baseqq;
					}



					double weight = invasc * invasc * intstep * intstep; //intstep*intstep
					double scaldefect = scal - (1. + invasc * mu1 * (xpt1 - tau1) + invasc * mu2 * (xpt2 - tau2));
					double scaldefect2 = scaldefect / Riemnorm;
					double diffcurv = Riemnorm * Riemnorm - scal * scal;

					sum = sum + Jac * det * weight;
					sum2 = sum2 + Jac * det * scaldefect * weight;
					sum3 = sum3 + Jac * det * (scaldefect) * (scaldefect) * weight;
					sum4 = sum4 + Jac * det * scaldefect * (xpt1 - kreal / 2.) * weight;
					sum5 = sum5 + Jac * det * scaldefect * (xpt2 - kreal / 2.) * weight;
					sum6 = sum6 + Jac * det * diffcurv * weight;
					
					/////////////////////////////////////
					//Results Plotting within main loop//
					/////////////////////////////////////
					if (plotResults)
					{
						if (!plotCutOff || plotCutOffCircle.contains(xpt1, xpt2))
						{
							double output = 0;
							switch(graphnumber)
							{
								case 1:
									output = scaldefect;
									break;
								case 2:
									output = bachsize;
									break;
								case 3:
									output = Riemnorm;
									break;
								case 4:
									output = weyl;
									break;
								case 5:
									output = tfricsize;
									break;
								case 6:
									output = gauss;
									break;
							}
							
							//store results for later plotting (for most values of graphnumber)
							plottingResults.push_back(Vector3(xpt1, xpt2, output));
							
							
							//plot sixcolourplot points
							if (libplotPlotting && plotType == 0)
							{
								switch(graphnumber)
								{
									case 1:
										sixcolourplot(scaldefect, xpt1, xpt2,
											-.06, -.03, -.01, .01, .03);
										break;
									case 2:
										sixcolourplot(bachsize, xpt1, xpt2,
											.001, .005, .01, .015, .02);
										break;
									case 3:
										sixcolourplot(Riemnorm, xpt1, xpt2,
											.5, 1., 2., 3., 5.);
										break;
									case 4:
										sixcolourplot(weyl, xpt1, xpt2,
											.1, .2, .5, 1, 2);
										break;
								}
							}
							
							//plot explanitory diagrams
							if (graphnumber >= 20)
							{
								switch(graphnumber)
								{
									case 21:
										if (alp == 0)
										{
											pl_fmarker(x1, x2, 2, .1);
										}
										else
										{
											pl_fmarker(x1, x2, 1, .1);
										}
										break;
									case 22:
										if (alp == 0)
										{
											pl_fmarker(z1, z2, 1, .03);
										}
										break;
									case 23:
										if (alp == 0)
										{
											pl_fmarker(xpt1, xpt2, 2, .1);
										}
										else
										{
											pl_fmarker(xpt1, xpt2, 1, .1);
										}
										break;
								}
							}
						}
					}
					
					//Calculate mins and maxes
					if (graphnumber == 20)
					{
						minMaxes[mm_defect].contend(scaldefect);
						minMaxes[mm_bach].contend(bachsize);
						minMaxes[mm_riemnorm].contend(Riemnorm);
						minMaxes[mm_weyl].contend(weyl);
						minMaxes[mm_ricci].contend(tfricsize);
						minMaxes[mm_gauss].contend(gauss);
					}
					
					
					//Calculate scalar defect max and min
					if (scaldefect > scaldefmax)
					{
						scaldefmax = scaldefect;
					}
					if (scaldefect < scaldefmin)
					{
						scaldefmin = scaldefect;
					}
					if (scaldefect2 > scaldef2max)
					{
						scaldef2max = scaldefect2;
					}
					if (scaldefect2 < scaldef2min)
					{
						scaldef2min = scaldefect2;
					}
					
					
					//recalculate powers, PINTEGRAL and SINTERGRAL
					double adjust = 4 * double(polynumber) / Area;
					
					for (int a = 0;a <= k;a++)
					{
						for (int b = 0;b <= k;b++)
						{
							if (chi[a][b])
							{
								PINTEGRAL[a][b] += Jac * det * weight 
													* powers[a][b] / F;
								SINTEGRAL[a][b] += adjust * B[a][b] * Jac * det 
										* weight * powers[a][b] * (scaldefect) / F;
								powers[a][b] = 0.;

							}
						}
					}


				}      //END MAIN LOOP
			}//END G-LOOP
		}//END F-LOOP

		double sumlcoff = 0.;
		double momcoff1 = 0.;
		double momcoff2 = 0.;

		for (int a = 0;a <= k;a++)
		{
			for (int b = 0;b <= k;b++)
			{
				if (chi[a][b])
				{

					double star = -log(PINTEGRAL[a][b]);
					//Lcoff[a][b] = star;
					sumlcoff = sumlcoff + star;
					momcoff1 = momcoff1 + (double(a) - kreal / 2.) * star;
					momcoff2 = momcoff2 + (double(b) - kreal / 2.) * star;
				}
			}
		}
		double muc = 3. - (sumlcoff / double(polynumber));
		double sig1 = kreal / 2. + Zmom1 / double(polynumber);
		double sig2 = kreal / 2. + Zmom2 / double(polynumber);
		momcoff1 = momcoff1 - sumlcoff * Zmom1 / double(polynumber);
		momcoff2 = momcoff2 - sumlcoff * Zmom2 / double(polynumber);
		double Zlam1 = (ZI12 * momcoff2 - ZI22 * momcoff1) / detZI;
		double Zlam2 = (ZI12 * momcoff1 - ZI11 * momcoff2) / detZI;
		double ratioab;

		newdel = 1;
		for (int a = 0;a <= k;a++)
		{
			for (int b = 0;b <= k;b++)
			{
				if (chi[a][b])
				{
					double logterm = muc + Zlam1 * (a - sig1) + Zlam2 * (b - sig2);
					double newB = exp(logterm) * (1 + delt[a][b]) / PINTEGRAL[a][b];
					rdelt[a][b] = (B[a][b] * PINTEGRAL[a][b] / (exp(logterm))) - 1.;
					ratioab = ((newB / B[a][b]) - 1);

					if (ratioab < 0)
					{
						ratioab = -ratioab;
					}
					if (ratioab > stopconstant2*kreal)
					{
						newdel = 0;
					}
					B[a][b] = newB;
					PINTEGRAL[a][b] = 0.;
				}
			}
		}
		
		//find min and max of B
		double sumcoff = 0.;
		double bmax = 0;
		double bmin = 10000;
		for (int a = 0;a <= k;a++)
		{
			for (int b = 0;b <= k;b++)
			{
				if (chi[a][b])
				{
					sumcoff = sumcoff + B[a][b];
					if (B[a][b] > bmax)
					{
						bmax = B[a][b];
					}
					if (B[a][b] < bmin)
					{
						bmin = B[a][b];
					}
				}
			}
		}
		
		
		////////////////////////////////////////////////////////////
		//Display interesting statistics about state of calculation/
		////////////////////////////////////////////////////////////
		cout << "****************" << endl;

		cout << "Iteration number: " << itnumber << endl;
		cout << "Runnumber: " << double(refnumber) + .001*double(runnumber) << endl;
		cout << "Max coefficient: " << bmax << endl;
		cout << "Min coefficient: " << bmin << endl;
		cout << "Normalised volume: " << 4. * sum / Area << endl; 
					//normalised volume--exact value 1

		cout << "Integral of scalar curvature defect " << 4.*sum2 / Area << endl;
		cout << "moments: " << 4. * sum4 / Area << endl;
		cout << 4. * sum5 / Area << endl;
		cout << "L2 difference: " << sum6*asc*asc << endl;
			//need to correct this 4(6-NV) exactble l2norm=sqrt(4.*sum3/Area);
		cout << "$$$" << endl;
		double l2norm = sqrt(4.*sum3 / Area);
		cout << "L2 norm of scalar curvature defect: " << l2norm << endl;
		cout << "max. value: " << scaldefmax << endl;
		cout << "min. value: " << scaldefmin << endl;
		cout << "max. value normalised: " << scaldef2max << endl;
		cout << "min. value normalised: " << scaldef2min << endl;
		cout.flush();
		
		//Display maxes and mins (if that output is selected)
		if (graphnumber == 20)
		{
			cout << endl;
			cout << "Defect:  " << minMaxes[mm_defect] << endl;
			cout << "Bach:  " << minMaxes[mm_bach] << endl;
			cout << "Riemnorm:  " << minMaxes[mm_riemnorm] << endl;
			cout << "Weyl:  " << minMaxes[mm_weyl] << endl;
			cout << "Ricci:  " << minMaxes[mm_ricci] << endl;
			cout << "Gauss:  " << minMaxes[mm_gauss] << endl;
		}
		
		///////////////////////////////////////////////
		//l2norms of scalar curvature defect plotting//
		///////////////////////////////////////////////
		//plot a point in libplot graph of l2norms of scalar curvature defects
		if (graphnumber == -1)
		{
			counter = counter + 1;
			double horiz = counter / 40.;//kreal*.002*double(counter);
			double vertic = graphver1 * log(l2norm) + graphver2; 
						//kreal*(.05*log(l2norm)+1.);
			pl_pencolorname("red");
			pl_fmarker(horiz, vertic, 3, .1);
			
		}
		
		//store an l2norm in a file (if this is enabled)
		if (storeL2Norm && !viewOnly)
		{
			ofstream l2normFile;
			l2normFile.open(l2normFileName.c_str(), ios_base::out | ios_base::app);
			l2normFile.precision(l2normPrecision);
			if (l2normFile.is_open())
			{
				l2normFile << l2norm << endl;
			}
			else
			{
				cout << "failed to open file to store l2norms" << endl;
				return 1;
			}
			l2normFile.close();
		}
		
		///////////////////////
		//plot simple contours/
		///////////////////////
		if (plotType == 2 && plotResults)
		{
			cout << "%%%%%%%%" << endl;
			simpleContourPlot(plottingResults, 3);
			//DEBUG - sleep(150);
		}
		
		////////////////////
		//plot colour plot//
		////////////////////
		if (plotType == 1 && plotResults)
		{
			cout << "%%%%%%%%" << endl;
			autoSixColourPlot(plottingResults);
		}
		
		//pause after display of results
		/*if (plotResults)
		{
			sleep(3);
		}*/
		
		
		
		////////////////////////
		//save results to file//
		////////////////////////
		
		//xyz file for xd3d plots
		//order of points randomised as this seems to reduce an xd3d plotting bug
		if (plotResults && plotType == 10)
		{
			cout << "Saving xyz file..." << endl;
			cout.flush();
			xyzSave(plottingResults, xyzFileName, xyzElimCloseness, true);
		}
		
		//csv file for matlab plots
		if (plotResults && plotType == 11)
		{
			cout << "Saving csv file..." << endl;
			cout.flush();
			xyzSave(plottingResults, xyzFileName, xyzElimCloseness, false, ',');
		}
		
		//plot graph to indicate points removed as too close
		if (plotResults && graphnumber == 30)
		{
			cout << "xyz close points elimination test." << endl;
			cout << "Total number of points: " << plottingResults.size() << endl;
				
			vector<Vector3> removedResults;
			removeClosePoints(plottingResults, xyzElimCloseness, removedResults);
			
			cout << "Removed points: " << removedResults.size() << endl;
			
			vector<Vector3>::iterator resEnd = removedResults.end();
			pl_pencolorname ("red");
			for(vector<Vector3>::iterator i = removedResults.begin(); i != resEnd; ++i)
			{
				pl_fmarker(i->getX(), i->getY(), 1, .001);
			}
		}
		
		//save the metric B
		if (!viewOnly && itnumber % saveInterval == 0)
		{
			resultsFile.save(B);
			resultsFile.save("Max coefficient: ", bmax);
			resultsFile.save("Min coefficient: ", bmin);
			resultsFile.save("L2 norm of scalar curvature defect", l2norm);
			resultsFile.save("Max scalar curvature defect", scaldefmax);
			resultsFile.save("Min scalar curvature defect", scaldefmin);
		}
		resultsFile.incrementNum();
	}//end of iteration loop
	
	cout << "Press enter to exit..." << endl;
	//cin.ignore(100, '\n'); //clean up new line character left in cin buffer by earlier uses of <<
	cin.get();
	
	
	return EXIT_SUCCESS;//always have this to show we have exited properly
}




bool inSquare(int p, int q, int k)
{
	return (0 <= p) && (p <= k) && (0 <= q) && (q <= k);
}

double XlogX(double x)
{
	double c = 0.;
	if (x > .001)
	{
		c = x * log(x);
	}
	return c;
}

double smallprod(int i, int a)
{
	double ireal = double(i); //convert to a double to avoid overflows
	switch(a)
	{
		case 0:
			return 1.;
		case 1:
			return ireal;
		case 2:
			return ireal*(ireal-1.);
		case 3:
			return ireal*(ireal-1.)*(ireal-2.);
		case 4:
			return ireal*(ireal-1.)*(ireal-2.)*(ireal-3.);
		case 5:
			return ireal*(ireal-1.)*(ireal-2.)*(ireal-3.)*(ireal-4.);
		default:
			cout << "error: smallprod used with invalid parameters. i: " << i << " a: " << a;
			return 1.;
	}
}


int cyclic(int r, int N, int i)
{
	if (i == -1)
	{
		if (r == 0)
		{
			return N - 1;
		}
		else
		{
			return r - 1;
		}
	}
	if (i == 1)
	{
		if (r == N - 1)
		{
			return 0;
		}
		else
		{
			return r + 1;
		}
	}
}

double lowpow(double y,int n)
{
	switch(n)
	{
		case 0:
			return 1.;
		case 1:
			return y;
		case -1:
			return powm1(y);
		case 2:
			return pow2(y);
		case -2:
			return powm2(y);
		case 3:
			return pow3(y);
		case -3:
			return powm3(y);
		case 4:
			return pow4(y);
		case -4:
			return powm4(y);
		case 5:
			return pow5(y);
		case -5:
			return powm5(y);
		default:
			cout << "error: lowpow used with invalid parameters. y: " << y << " n: " << n;
			return 1.;
	}
}



void skipline(ifstream & mystream)
{
   mystream.ignore(1000,'\n');
}




