
#ifndef MATHS_UTIL_H
#define MATHS_UTIL_H

#include <cmath>
#include <limits>
#include <iostream>

//Positive powers
//! @return x to the power of 2
template<class T> inline T pow2(T x) {return x*x;}
//! @return x to the power of 3
template<class T> inline T pow3(T x) {return x*x*x;}
//! @return x to the power of 4
template<class T> inline T pow4(T x) {return pow2(pow2(x));}
//! @return x to the power of 5
template<class T> inline T pow5(T x) {return pow4(x)*x;}
//! @return x to the power of 6
template<class T> inline T pow6(T x) {T x2 = pow2(x); return pow2(x2)*x2;}
//! @return x to the power of 7
template<class T> inline T pow7(T x) {return pow6(x)*x;}
//! @return x to the power of 8
template<class T> inline T pow8(T x) {return pow2(pow4(x));}
//! @return x to the power of 9
template<class T> inline T pow9(T x) {return pow8(x)*x;}

//Negative powers
//! @return the reciprocal of x
template<class T> inline T recip(T x) {return 1./x;}
//! @return x to the power of -1
template<class T> inline T powm1(T x) {return 1./x;}
//! @return x to the power of -2
template<class T> inline T powm2(T x) {return 1./pow2(x);}
//! @return x to the power of -3
template<class T> inline T powm3(T x) {return 1./pow3(x);}
//! @return x to the power of -4
template<class T> inline T powm4(T x) {return 1./pow4(x);}
//! @return x to the power of -5
template<class T> inline T powm5(T x) {return 1./pow5(x);}
//! @return x to the power of -6
template<class T> inline T powm6(T x) {return 1./pow6(x);}
//! @return x to the power of -7
template<class T> inline T powm7(T x) {return 1./pow7(x);}
//! @return x to the power of -8
template<class T> inline T powm8(T x) {return 1./pow8(x);}
//! @return x to the power of -9
template<class T> inline T powm9(T x) {return 1./pow9(x);}



/**
* @return whether n is exactly a power of 2
*/
inline bool isPow2(unsigned long n)
{
	return !(bool)(n & (n - 1));
}


/**
* A class to ease finding the maximum and minimum of a set of doubles.
* Potentially useful for storing a running max and min as the set is generated.
*/
class MinMax
{
	public:
		MinMax(): m_min(std::numeric_limits<double>::infinity()), m_max(-std::numeric_limits<double>::infinity()) {}
		~MinMax() {}
		
		//! present x as an element of the set to be compared to the current min and max
		inline void contend(double x)
		{
			if (x < m_min)
			{
				m_min = x;
			}
			if (x > m_max)
			{
				m_max = x;
			}
		}
		
		//! present x as an element of the set to be compared to the current min and max - discounting x if it is +inf or -inf
		inline void contendNI(double x)
		{
			if (!std::isinf(x))
			{
				contend(x);
			}
		}
		
		//! @return whether any items other than nan have been contended
		inline bool empty()
		{
			return m_min == std::numeric_limits<double>::infinity() 
					&& m_max == -std::numeric_limits<double>::infinity();
		}
		
		//! @return whether the MinMax is in a valid, non-empty state with neither max nor min nan
		inline bool valid()
		{
			return m_min <= m_max;
		}
		
		//! output the stored max and min to an output stream
		inline void display(std::ostream & out) const
		{
			out << "Max:" << m_max << " ";
			out << "Min:" << m_min;
		}
		
		
		
		inline double min() const {return m_min;}
		inline double max() const {return m_max;}
	private:
		double m_min;
		double m_max;
};

inline std::ostream & operator<< (std::ostream & out, const MinMax & mm)
{
	mm.display(out);
	return out;
}

#endif //MATHS_UTIL_H
