

//A file of utility functions for using libplot
#ifndef PLOT_UTIL_H
#define PLOT_UTIL_H

#include <stdio.h> //required by plot.h
#include <plot.h>
#include <string>

/**
* Set up a libplot plotter for use with the libplot C bindings.
* On failure an error message is displayed to stderr and the program forced to exit.
* @param display_type the type of plotter output - for example "X" to plot to a window, "png" to plot to a png file
* @param size the size of the plot area 
*			- if this value starts with a number, for example "300x200", the bitmap plotsize is set
*			- if this value does not start with a number, for example "letter", the postscript page size is set
* @param file a pointer to which image data should be written (if the plotter is not to be a window plotter)
* @return the handle of the plotter created
*/
int init_plotsession(const char* display_type, const char* size, FILE* file = NULL);
/**
* Clear up a libplot plotter.
* On failure, an error message is displayed to stderr.
* @param handle the handle of the plotter to close
* @return an error code - 0 if the plotter was closed successfully
*/
int end_plotsession(int handle);

/* 
//Example code - opening a plotter to save to file
	string fileName = "myFile.png";
	char * display_type = "png";
	char * size = "800x700";
	FILE * file = fopen(fileName.c_str(), "w");
	if (!file)
	{
		cout << "failed to open file to save plot" << endl;
		exit(1);
	}
	int handle = init_plotsession(display_type, size, file);
	pl_space (-1, -1, 1, 1);

//Example code - closing a plotter have used to save to file
	end_plotsession(handle);
	if (file)
	{
		int ret = fclose(file);
		if (ret != 0)
		{
			cout << "failed to close picture file" << endl;
		}
	}
*/
	


const int COLOURMAX = 65535; //highest value a libplot colour value may take


/**
* A class to ease the use of libplot plotters with the libplot C bindings.
* The object manages the plotter handle and the file to which the plotter writes (if relevant) closing them automatically 
* when the object goes out of scope.
* The class displays error messages, and if a plotter cannot be opened it forces the program to exit.
* Example usage:
* 	CPlotter myPlotter;
* 	myPlotter.open("png", "300x350", "myfile");
* 	pl_space (-1, -1, 1, 1);
* This is a pretty odd idea really, and the C++ libplot bindings and class Plotter should probably be preferred.
*/
class CPlotter
{
	public:
		//! Creates CPlotters in an empty state managing no libplot plotter.
		CPlotter(): m_handle(-1), m_file(NULL) {}
		//! Closes any currently managed plotter, and transfers that from another CPlotter, leaving it in an empty state
		CPlotter(CPlotter & other);
		~CPlotter();
		
		/**
		 * Set up a libplot plotter managed by this CPlotter.
		 * On failure an error message is displayed to stderr and the program forced to exit.
		 * @param display_type the type of plotter output - for example "X" to plot to a window, "png" to plot to a png file
		 * @param size the size of the plot area 
		 *			- if this value starts with a number, for example "300x200", the bitmap plotsize is set
		 *			- if this value does not start with a number, for example "letter", the postscript page size is set
		 * @param fileName The path at which to save the file of image data (if the plotter is not to be a window plotter).
		 * 		The display_type is appended to this name as a file extension.
		 */
		void open(const std::string & display_type, const std::string & size, const std::string & fileName = "");
		/**
		 * Set up a libplot plotter managed by this CPlotter with a default size for each plotter type.
		 * On failure an error message is displayed to stderr and the program forced to exit.
		 * @param displayTypeChar character specifying type and size of plotter output:
		 * 		- x		X window 600x600
		 *		- p 	png 600x600
		 * 		- g		gif 600x600
		 * 		- s		postscript A4 page
		 * @param fileName The path at which to save the file of image data (if the plotter is not to be a window plotter).
		 * 		An appropriate file extension is appended to this name.
		 */
		void open(char displayTypeChar, const std::string & fileName = "");
		/**
		* Close the managed libplot plotter.
		* @return whether the command succeded.
		*/
		bool close();
		/**
		 * Select the managed libplot plotter as the active plotter.
		 * @return whether the command succeded.
		 */
		bool select() const;
		/**
		* Display to stdout details of the managed plotter output type and size.
		*/
		void printDetails() const;
		
		int getHandle() const;
		FILE * getFile() const;
		const std::string & getDisplayType() const;
		const std::string & getSize() const;
		const std::string & getFileName() const;
	private:
		int m_handle;
		FILE * m_file;
		std::string m_displayType;
		std::string m_size;
		std::string m_fileName;
};
#endif //PLOT_UTIL_H
